'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.totalSupply = totalSupply;
exports.balanceOf = balanceOf;
exports.transfer = transfer;
exports.Transfer = Transfer;
exports.allEvents = allEvents;
exports.getPastEvents = getPastEvents;
exports.estimateGas = estimateGas;

var _contractMethod = require('./contractMethod');

var _contractMethod2 = _interopRequireDefault(_contractMethod);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var interfaceName = 'ERC20';

/**
 * @async
 * @function totalSupply
 *
 * @description Gets total supply of tokens
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.contractAddress - Contract address
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the total supply of tokens
 */
/**
 * @file Exposes ERC20 API
 * @author Ivan Violentov <ivan.violentov@jibrel.network>
 */

function totalSupply(props) {
  return _contractMethod2.default.call({ props: props, interfaceName: interfaceName, method: 'totalSupply', args: [] });
}

/**
 * @async
 * @function balanceOf
 *
 * @description Gets balance of specific address
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.contractAddress - Contract address
 * @param {string} props.owner - Address to check balance
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the balance of owner address
 */
function balanceOf() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  return _contractMethod2.default.call({ props: props, interfaceName: interfaceName, method: 'balanceOf', args: [props.owner] });
}

/**
 * @async
 * @function transfer
 *
 * @description Transfers tokens
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.contractAddress - Contract address
 * @param {string} props.privateKey - Private key (64 hex symbols, without '0x' prefix)
 * @param {string} props.to - Address of tokens receiver
 * @param {BigNumber} props.value - Transaction value
 * @param {BigNumber} [props.gasLimit] - Gas limit for the transaction
 * @param {BigNumber} [props.gasPrice] - Gas price for the transaction
 * @param {number} [props.nonce] - Nonce for the transaction
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the hash of created contract transaction
 */
function transfer() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var to = props.to,
      value = props.value;


  return _contractMethod2.default.sendTransaction({
    props: props,
    interfaceName: interfaceName,
    method: 'transfer',
    args: [to, value]
  });
}

/**
 * @async
 * @function Transfer
 *
 * @description Subscribes to Transfer event
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.contractAddress - Contract address
 * @param {object} [props.options] - Event options (@see subscribe)
 * @param {function} [props.callback] - Event callback (@see subscribe)
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the event emitter (@see subsribe)
 */
function Transfer(props) {
  return _contractMethod2.default.subscribeToEvent({ props: props, interfaceName: interfaceName, method: 'Transfer' });
}

/**
 * @async
 * @function allEvents
 *
 * @description Subscribes to all contract events
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.contractAddress - Contract address
 * @param {object} [props.options] - Event options (@see subscribe)
 * @param {function} [props.callback] - Event callback (@see subscribe)
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the event emitter (@see subsribe)
 */
function allEvents(props) {
  return _contractMethod2.default.subscribeToEvent({ props: props, interfaceName: interfaceName, method: 'allEvents' });
}

/**
 * @async
 * @function getPastEvents
 *
 * @description Gets past events
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.contractAddress - Contract address
 * @param {string} props.event - Event name (@see subscribe)
 * @param {object} [props.options] - Event options (@see subscribe)
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the event logs (@see getPast)
 */
function getPastEvents(props) {
  return _contractMethod2.default.getPastEvents({ props: props, interfaceName: interfaceName, method: 'getPastEvents' });
}

/**
 * @async
 * @function estimateGas
 *
 * @description Gets estimate gas for the contract transaction
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.contractAddress - Contract address
 * @param {string} props.privateKey - Private key (64 hex symbols, without '0x' prefix)
 * @param {string} props.method - Method name
 * @param {array} props.args - Method arguments
 *
 * @returns Promise that will be resolved with estimate gas value
 */
function estimateGas(props) {
  return _contractMethod2.default.estimateGas({ props: props, interfaceName: interfaceName, method: 'estimateGas' });
}

exports.default = { totalSupply: totalSupply, balanceOf: balanceOf, transfer: transfer, Transfer: Transfer, allEvents: allEvents, getPastEvents: getPastEvents, estimateGas: estimateGas };