'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isReceivingAllowed = isReceivingAllowed;
exports.isSpendingAllowed = isSpendingAllowed;
exports.isTransferAllowed = isTransferAllowed;
exports.isApproveAllowed = isApproveAllowed;
exports.isApprovedSpendingAllowed = isApprovedSpendingAllowed;
exports.isTransferFromAllowed = isTransferFromAllowed;
exports.estimateGas = estimateGas;

var _contractMethod = require('./contractMethod');

var _contractMethod2 = _interopRequireDefault(_contractMethod);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var interfaceName = 'ERC20Validatable';

/**
 * @async
 * @function isReceivingAllowed
 *
 * @description Gets isReceivingAllowed flag
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.contractAddress - Contract address
 * @param {string} props.account - Account address
 * @param {BigNumber} props.value - Transaction value
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the isReceivingAllowed flag
 */
/**
 * @file Exposes ERC20Validatable API
 * @author Ivan Violentov <ivan.violentov@jibrel.network>
 */

function isReceivingAllowed(props) {
  var account = props.account,
      value = props.value;


  return _contractMethod2.default.call({
    props: props,
    interfaceName: interfaceName,
    method: 'isReceivingAllowed',
    args: [account, value]
  });
}

/**
 * @async
 * @function isSpendingAllowed
 *
 * @description Gets isSpendingAllowed flag
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.contractAddress - Contract address
 * @param {string} props.account - Account address
 * @param {BigNumber} props.value - Transaction value
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the isSpendingAllowed flag
 */
function isSpendingAllowed(props) {
  var account = props.account,
      value = props.value;


  return _contractMethod2.default.call({
    props: props,
    interfaceName: interfaceName,
    method: 'isSpendingAllowed',
    args: [account, value]
  });
}

/**
 * @async
 * @function isTransferAllowed
 *
 * @description Gets isTransferAllowed flag
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.contractAddress - Contract address
 * @param {string} props.from - Sender address
 * @param {string} props.to - Receiver address
 * @param {BigNumber} props.value - Transaction value
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the isTransferAllowed flag
 */
function isTransferAllowed(props) {
  var from = props.from,
      to = props.to,
      value = props.value;


  return _contractMethod2.default.call({
    props: props,
    interfaceName: interfaceName,
    method: 'isTransferAllowed',
    args: [from, to, value]
  });
}

/**
 * @async
 * @function isApproveAllowed
 *
 * @description Gets isApproveAllowed flag
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.contractAddress - Contract address
 * @param {string} props.from - Sender address
 * @param {string} props.spender - Spender address
 * @param {BigNumber} props.value - Transaction value
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the isApproveAllowed flag
 */
function isApproveAllowed(props) {
  var from = props.from,
      spender = props.spender,
      value = props.value;


  return _contractMethod2.default.call({
    props: props,
    interfaceName: interfaceName,
    method: 'isApproveAllowed',
    args: [from, spender, value]
  });
}

/**
 * @async
 * @function isApprovedSpendingAllowed
 *
 * @description Gets isApprovedSpendingAllowed flag
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.contractAddress - Contract address
 * @param {string} props.from - Sender address
 * @param {string} props.spender - Spender address
 * @param {BigNumber} props.value - Transaction value
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the isApprovedSpendingAllowed flag
 */
function isApprovedSpendingAllowed(props) {
  var from = props.from,
      spender = props.spender,
      value = props.value;


  return _contractMethod2.default.call({
    props: props,
    interfaceName: interfaceName,
    method: 'isApprovedSpendingAllowed',
    args: [from, spender, value]
  });
}

/**
 * @async
 * @function isTransferFromAllowed
 *
 * @description Gets isTransferFromAllowed flag
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.contractAddress - Contract address
 * @param {string} props.spender - Spender address
 * @param {string} props.from - Sender address
 * @param {string} props.to - Receiver address
 * @param {BigNumber} props.value - Transaction value
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the isTransferFromAllowed flag
 */
function isTransferFromAllowed(props) {
  var spender = props.spender,
      from = props.from,
      to = props.to,
      value = props.value;


  return _contractMethod2.default.call({
    props: props,
    interfaceName: interfaceName,
    method: 'isTransferFromAllowed',
    args: [spender, from, to, value]
  });
}

/**
 * @async
 * @function estimateGas
 *
 * @description Gets estimate gas for the contract transaction
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.contractAddress - Contract address
 * @param {string} props.privateKey - Private key (64 hex symbols, without '0x' prefix)
 * @param {string} props.method - Method name
 * @param {array} props.args - Method arguments
 *
 * @returns Promise that will be resolved with estimate gas value
 */
function estimateGas(props) {
  return _contractMethod2.default.estimateGas({ props: props, interfaceName: interfaceName, method: 'estimateGas' });
}

exports.default = {
  isReceivingAllowed: isReceivingAllowed,
  isSpendingAllowed: isSpendingAllowed,
  isTransferAllowed: isTransferAllowed,
  isApproveAllowed: isApproveAllowed,
  isApprovedSpendingAllowed: isApprovedSpendingAllowed,
  isTransferFromAllowed: isTransferFromAllowed,
  estimateGas: estimateGas
};