'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sendTransaction = sendTransaction;
exports.getBalance = getBalance;
exports.getBlockNumber = getBlockNumber;
exports.getBlock = getBlock;
exports.getTransaction = getTransaction;
exports.getTransactionReceipt = getTransactionReceipt;
exports.getLogsFilter = getLogsFilter;
exports.getPastLogs = getPastLogs;
exports.estimateGas = estimateGas;

var _ethMethod = require('./ethMethod');

var _ethMethod2 = _interopRequireDefault(_ethMethod);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var interfaceName = 'ETH';

/**
 * @async
 * @function sendTransaction
 *
 * @description Sends transaction
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.privateKey - Private key (64 hex symbols, without '0x' prefix)
 * @param {string} props.to - Address of the transaction receiver
 * @param {BigNumber} props.value - Transaction value (in wei)
 * @param {BigNumber} [props.gasLimit] - Gas limit for the transaction
 * @param {BigNumber} [props.gasPrice] - Gas price for the transaction
 * @param {number} [props.nonce] - Nonce for the transaction
 * @param {string} [props.data] - Transaction data
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the hash of created transaction
 */
/**
 * @file Exposes ETH API
 * @author Ivan Violentov <ivan.violentov@jibrel.network>
 */

function sendTransaction(props) {
  return _ethMethod2.default.sendTransaction({ props: props, interfaceName: interfaceName, method: 'sendTransaction' });
}

/**
 * @async
 * @function getBalance
 *
 * @description Gets balance of the provided address
 *
 * @param {object} props={} - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.address - Address to check balance
 * @param {(number|string)} [props.defaultBlock] - Redefines of web3.eth.defaultBlock
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with balance of the provided address
 */
function getBalance() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var address = props.address,
      defaultBlock = props.defaultBlock;


  return _ethMethod2.default.call({
    props: props,
    interfaceName: interfaceName,
    method: 'getBalance',
    args: [address, defaultBlock]
  });
}

/**
 * @async
 * @function getBlockNumber
 *
 * @description Gets current block number
 *
 * @param {object} props={} - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with current block number
 */
function getBlockNumber() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  return _ethMethod2.default.call({
    props: props,
    interfaceName: interfaceName,
    method: 'getBlockNumber',
    args: []
  });
}

/**
 * @async
 * @function getBlock
 *
 * @description Returns block data
 *
 * @param {object} props={} - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} [props.blockId] - block number or hash. Or the string "earliest"/"latest"/"pending"
 * @param {boolean} [props.returnTransactionObjects] - return all transactions as objects if true
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with block data
 */
function getBlock() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var blockId = props.blockId,
      returnTransactionObjects = props.returnTransactionObjects;


  return _ethMethod2.default.call({
    props: props,
    interfaceName: interfaceName,
    method: 'getBlock',
    args: [blockId || 'latest', returnTransactionObjects]
  });
}

/**
 * @async
 * @function getTransaction
 *
 * @description Returns transaction data
 *
 * @param {object} props={} - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.transactionHash - Transaction hash
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with transaction data
 */
function getTransaction() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  return _ethMethod2.default.call({
    props: props,
    interfaceName: interfaceName,
    method: 'getTransaction',
    args: [props.transactionHash]
  });
}

/**
 * @async
 * @function getTransactionReceipt
 *
 * @description Returns the receipt of a transaction
 *
 * @param {object} props={} - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.transactionHash - Transaction hash
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with transaction receipt
 */
function getTransactionReceipt() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  return _ethMethod2.default.call({
    props: props,
    interfaceName: interfaceName,
    method: 'getTransactionReceipt',
    args: [props.transactionHash]
  });
}

/**
 * @async
 * @function getLogsFilter
 *
 * @description Returns object for filtering of logs
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {object} [props.options] - Filter options (@see filter)
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the filter object (@see filter)
 */
function getLogsFilter(props) {
  return _ethMethod2.default.filterLogs({ props: props, interfaceName: interfaceName, method: 'getLogsFilter' });
}

/**
 * @async
 * @function getPastLogs
 *
 * @description Gets past events
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {object} [props.options] - Logs options (@see filter)
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with the event logs (@see getLogs)
 */
function getPastLogs(props) {
  return _ethMethod2.default.getPastLogs({ props: props, interfaceName: interfaceName, method: 'getPastLogs' });
}

/**
 * @async
 * @function estimateGas
 *
 * @description Gets estimate gas for the transaction
 *
 * @param {object} props - API function properties
 * @param {string} props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} props.rpcport - RPC port of Ethereum node to connect on
 * @param {string} props.to - Address of the transaction receiver
 * @param {BigNumber} props.value - Transaction value
 * @param {string} [props.data] - Transaction data
 * @param {boolean} [props.ssl] - Defines using of ssl for connection or not
 *
 * @returns Promise that will be resolved with estimate gas value
 */
function estimateGas(props) {
  return _ethMethod2.default.estimateGas({ props: props, interfaceName: interfaceName, method: 'estimateGas' });
}

exports.default = {
  sendTransaction: sendTransaction,
  getBalance: getBalance,
  getBlockNumber: getBlockNumber,
  getBlock: getBlock,
  getTransaction: getTransaction,
  getTransactionReceipt: getTransactionReceipt,
  getLogsFilter: getLogsFilter,
  getPastLogs: getPastLogs,
  estimateGas: estimateGas
};