'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

exports.subscribe = subscribe;
exports.getEvents = getEvents;

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _config = require('../config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @callback eventCallback
 *
 * @param {Object} error
 * @param {Object} event
 */

/**
 * @function subscribe
 *
 * @description Subscribes to provided event
 *
 * @param {function} Event - Function for event subscribing
 * @param {object} [options={}] - Event options
 * @param {object} [options.filter] - Filter options by indexed event parameters
 * @param {(number|string)} [options.fromBlock] - The number of the earliest block
 * @param {(number|string)} [options.toBlock] - The number of the latest block
 * @param {(string|string[])} [options.address] - An address(es) to get logs from
 * @param {string[]} [options.topics] - Allows to manually set the topics for the event filter
 * @param {eventCallback} [callback] - Callback which fired for each event or error
 *
 * @returns {object} The event emitter has the following events:<br />
 * - data: Fires on each incoming event with the event object as argument<br />
 * - error: Fires when an error in the subscription occours
 */
function subscribe(Event) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var callback = arguments[2];

  var eventEmitter = new _events2.default();

  /**
   * web3@0.x.x event takes filter and additional options in different params
   * web3@1.x.x event takes all options in one param
   */
  var filter = options.filter,
      additionalOptions = (0, _objectWithoutProperties3.default)(options, ['filter']);


  Event(filter, additionalOptions, function (err, result) {
    if (err) {
      /**
       * Error event
       *
       * @event subscribeErrorEvent
       * @type {object}
       */
      eventEmitter.emit('error', err);

      if (callback) {
        callback(err);
      }

      return;
    }

    /**
     * Data event
     *
     * @event subscribeDataEvent
     * @type {object}
     */
    eventEmitter.emit('data', result);

    if (callback) {
      callback(null, result);
    }
  });

  return eventEmitter;
}

/**
 * @async
 * @function getEvents
 *
 * @description Gets past events
 *
 * @param {function} Event - Function to get past events
 * @param {object} [options={}] - Event options
 * @param {object} [options.filter] - Filter options by indexed event parameters
 * @param {(number|string)} [options.fromBlock] - The number of the earliest block
 * @param {(number|string)} [options.toBlock] - The number of the latest block
 * @param {(string|string[])} [options.address] - An address(es) to get logs from
 * @param {string[]} [options.topics] - Allows to manually set the topics for the event filter
 *
 * @returns Promise that will be resolved with past events
 */
/**
 * @file Manages helper functions to work with events
 * @author Ivan Violentov <ivan.violentov@jibrel.network>
 */

function getEvents(Event) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  /**
   * web3@0.x.x event takes filter and additional options in different params
   * web3@1.x.x event takes all options in one param
   */
  var filter = options.filter,
      additionalOptions = (0, _objectWithoutProperties3.default)(options, ['filter']);

  var event = Event(filter, additionalOptions);

  /**
   * event.get uses instance methods inside,
   * but bluebird promisify don't save context,
   * so need to bind to event object directly
   */
  return _bluebird2.default.promisify(event.get.bind(event))().timeout(_config2.default.promiseTimeout, new Error('Can not get past events within ' + _config2.default.promiseTimeout + 'ms'));
}